package com.bonree.sdk.bonree_flutter_plugin;

import com.bonree.sdk.agent.Bonree;
import com.bonree.sdk.agent.engine.external.BonreeFlutterBridge;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * BonreeFlutterPlugin
 */
public class BonreeFlutterPlugin implements FlutterPlugin, MethodCallHandler {

    private MethodChannel channel;

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "bonree_flutter_plugin");
        channel.setMethodCallHandler(this);
    }

    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "bonree_flutter_plugin");
        channel.setMethodCallHandler(new BonreeFlutterPlugin());
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        try {
            if ("bonree_reportCrash".equals(call.method)) {
                BonreeFlutterBridge.reportCrash((long) call.argument("time"), (String) call.argument("errorValue"), (String) call.argument("reason"), (String) call.argument("stacktrace"));
            } else if ("bonree_reportView".equals(call.method)) {
                long time = call.argument("time");
                String viewId = call.argument("viewId");
                int loadTime = call.argument("loadTime");
                int model = call.argument("model");
                String name = call.argument("name");
                String method = call.argument("method");
                BonreeFlutterBridge.reportFlutterView(time, viewId, loadTime, model, name, method);
            } else if ("bonree_setCustomException".equals(call.method)) {
                Bonree.setCustomException((String) call.argument("exceptionType"), (String) call.argument("causedBy"), (String) call.argument("errorDump"));
            } else if ("bonree_setCustomPageStart".equals(call.method)) {
                Bonree.setCustomPageStart((String) call.argument("pageName"), (String) call.argument("pageAlias"));
            } else if ("bonree_setCustomPageEnd".equals(call.method)) {
                Bonree.setCustomPageEnd((String) call.argument("pageName"), (String) call.argument("pageAlias"));
            } else if ("bonree_setCustomEvent".equals(call.method)) {
                Bonree.setCustomEvent((String) call.argument("eventID"), (String) call.argument("eventName"), (String) call.argument("param"));
            } else if ("bonree_setCustomLog".equals(call.method)) {
                Bonree.setCustomLog((String) call.argument("logInfo"), (String) call.argument("param"));
            } else if ("bonree_setCustomMetric".equals(call.method)) {
                Object obj = call.argument("metricValue");
                if (obj instanceof Integer) {
                    Integer value = (Integer) obj;
                    Bonree.setCustomMetric((String) call.argument("metricName"), value.longValue(), (String) call.argument("param"));
                } else if (obj instanceof Long) {
                    Long value = (Long) obj;
                    Bonree.setCustomMetric((String) call.argument("metricName"), value, (String) call.argument("param"));
                }
            } else if ("bonree_setCustomMethodStart".equals(call.method)) {
                Bonree.setCustomMethodStart((String) call.argument("methodName"), (String) call.argument("param"));
            } else if ("bonree_setCustomMethodEnd".equals(call.method)) {
                Bonree.setCustomMethodEnd((String) call.argument("methodName"), (String) call.argument("param"));
            } else if ("bonree_setUserID".equals(call.method)) {
                String id = call.argument("userID");
                Bonree.setUserID(id);
            } else if ("bonree_setExtraInfo".equals(call.method)) {
                Map<String, Object> map = call.arguments();
                Bonree.setExtraInfo(map);
            } else if ("bonree_recordCustomLaunchEnd".equals(call.method)) {
                Bonree.recordCustomLaunchEnd();
            } else if ("bonree_reportNetwork".equals(call.method)) {
                android.util.Log.e("TAG", "reportNetwork: responseDataSize=" + call.argument("responseDataSize"));
                BonreeFlutterBridge.reportNetwork((String) call.argument("requestUrl"), (long) call.argument("startTimeMs"),
                        (int) call.argument("responseDataSize"), (String) call.argument("method"),
                        (String) call.argument("resourceType"), (int) call.argument("connectTimeMs"),
                        (int) call.argument("errorCode"), (int) call.argument("responseTimeMs"),
                        (String) call.argument("errorMessage"), (Map) call.argument("httpRequestHeader"),
                        (Map) call.argument("httpResponseHeader"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
