#import "BonreeFlutterPlugin.h"
#import <BonreeSDK/BonreeSDK.h>

@implementation BonreeFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {

    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"bonree_flutter_plugin" binaryMessenger:[registrar messenger]];
    BonreeFlutterPlugin *instatance = [[BonreeFlutterPlugin alloc]
                             init];
    [registrar addMethodCallDelegate:instatance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    //网络
    if ([@"bonree_reportNetwork" isEqualToString:call.method]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            BRSFlutterNetworkModel *netModel = [[BRSFlutterNetworkModel alloc] init];
            [netModel setValuesForKeysWithDictionary:call.arguments];
            //
            [BRSAgent flutterReportNetwork:[BonreeFlutterPlugin transformFlutterModelToCusTomModel:netModel]];

        }//崩溃
    } else if ([call.method isEqualToString:@"bonree_reportCrash"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent flutterReportErrorWithTime:[call.arguments[@"time"] longLongValue] causeby:call.arguments[@"reason"] type:call.arguments[@"errorValue"] dump:call.arguments[@"stacktrace"]];

        } //视图
    } else if ([call.method isEqualToString:@"bonree_reportView"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent flutterReportViewWithTime:[call.arguments[@"time"] longLongValue] viewId:call.arguments[@"viewId"] loadTime:[call.arguments[@"loadTime"] integerValue] model:[call.arguments[@"model"] intValue] name:call.arguments[@"name"] methodName:call.arguments[@"method"]];
        }
        //自定义
    } else if ([call.method isEqualToString:@"bonree_setCustomException"]) {
        
        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomExceptionWithType:call.arguments[@"exceptionType"] causeBy:call.arguments[@"causedBy"] errorDump:call.arguments[@"errorDump"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomPageStart"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomPageStartWithName:call.arguments[@"pageName"] pageAlias:call.arguments[@"pageAlias"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomPageEnd"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomPageEndWithName:call.arguments[@"pageName"] pageAlias:call.arguments[@"pageAlias"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomEvent"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomEventWithID:call.arguments[@"eventID"] name:call.arguments[@"eventName"] param:call.arguments[@"param"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomLog"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomLog:call.arguments[@"logInfo"] param:call.arguments[@"param"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomMetric"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomMetricWithName:call.arguments[@"metricName"] value:[call.arguments[@"metricValue"] integerValue] param:call.arguments[@"param"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomMethodStart"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomMethodStartWithName:call.arguments[@"methodName"] param:call.arguments[@"param"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setCustomMethodEnd"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setCustomMethodEndWithName:call.arguments[@"methodName"] param:call.arguments[@"param"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setUserID"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setUserID:call.arguments[@"userID"]];
        }
    } else if ([call.method isEqualToString:@"bonree_setExtraInfo"]) {

        if ([call.arguments isKindOfClass:[NSDictionary class]]) {

            [BRSAgent setExtraInfo:call.arguments];
        }
    } else if ([call.method isEqualToString:@"bonree_recordCustomLaunchEnd"]) {

            [BRSAgent recordCustomLaunchEnd];
    }

}

+ (BRSNetworkModel *)transformFlutterModelToCusTomModel:(BRSFlutterNetworkModel *)model {
    BRSNetworkModel *networkModel = [[BRSNetworkModel alloc] init];

    networkModel.requestUrl = [BonreeFlutterPlugin isNoNullAndBlankStr:model.requestUrl] ? model.requestUrl : nil;
    networkModel.responseDataSize = [model.responseDataSize integerValue] > 0 ? [model.responseDataSize unsignedIntValue]: 0;
    networkModel.method = [BonreeFlutterPlugin isNoNullAndBlankStr:model.method] > 0 ? model.method : nil;
    networkModel.resourceType = [BonreeFlutterPlugin isNoNullAndBlankStr:model.resourceType] ? model.resourceType : nil;
    networkModel.connectTimeUs = [model.connectTimeMs integerValue] > 0 ? [model.connectTimeMs longLongValue] * 1000 : 0;
    networkModel.errorCode = [model.errorCode integerValue] != -1 ? model.errorCode : nil;
    networkModel.responseTimeUs = [model.responseTimeMs integerValue] > 0 ? [model.responseTimeMs longLongValue] * 1000 : 0;
    networkModel.errorMessage = [BonreeFlutterPlugin isNoNullAndBlankStr:model.errorMessage] ? model.errorMessage : nil;
    networkModel.requestHeader = model.httpRequestHeader.allKeys.count > 0 ? [model.httpRequestHeader mutableCopy] : nil;
    networkModel.responseHeader = model.httpResponseHeader.allKeys.count > 0 ? [model.httpResponseHeader mutableCopy] : nil;

    return networkModel;
}

+ (BOOL)isNoNullAndBlankStr:(NSString *)str{

    return str.length > 0 && ![str isEqualToString:@"null"];
}

@end

@implementation BRSFlutterNetworkModel

-(void)setValue:(id)value forUndefinedKey:(NSString *)key{

}

@end
