import 'dart:io';
import 'dart:math';
import '../../business/bonree_business_interface.dart';
import 'bonree_http_request.dart';

class BonreeHttpClient implements HttpClient {
  HttpClient _httpClient;
  BonreeBusinessInterface _bonreeBusinessInterface;

  BonreeHttpClient(this._httpClient, this._bonreeBusinessInterface);

  Future<HttpClientRequest> _wrapRequest(
      HttpClientRequest httpClientRequest, String requestId) {
    return Future.value(BonreeHttpRequest(
        httpClientRequest, _bonreeBusinessInterface, requestId));
  }

  @override
  void close({bool force = false}) => _httpClient.close(force: force);

  @override
  Future<HttpClientRequest> delete(String host, int port, String path) async {
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request = await _httpClient.delete(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> deleteUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.deleteUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> getUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.getUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> head(String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      String url = Uri(host: host, port: port, path: path).toString();
      _bonreeBusinessInterface.startWebRequest(requestkey, url);
      HttpClientRequest request = await _httpClient.head(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);

      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> headUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.headUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);

      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> open(
      String method, String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request =
          await _httpClient.open(method, host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> openUrl(String method, Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.openUrl(method, url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> patch(String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request = await _httpClient.patch(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> patchUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.patchUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> post(String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request = await _httpClient.post(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> postUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.postUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> put(String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request = await _httpClient.put(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> putUrl(Uri url) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(requestkey, url.toString());
      HttpClientRequest request = await _httpClient.putUrl(url);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  @override
  Future<HttpClientRequest> get(String host, int port, String path) async {
    //随机key
    String requestkey = new DateTime.now().microsecondsSinceEpoch.toString() +
        new Random().nextInt(1000).toString();
    try {
      _bonreeBusinessInterface.startWebRequest(
          requestkey, Uri(host: host, port: port, path: path).toString());
      HttpClientRequest request = await _httpClient.get(host, port, path);
      _bonreeBusinessInterface.endWebRequest(
          requestkey, request.method, request.headers);
      return _wrapRequest(request, requestkey);
    } catch (error) {
      _bonreeBusinessInterface.finishWebRequset(requestkey, error.toString());
      throw error;
    }
  }

  // NOT USED FUNCTIONS

  @override
  bool get autoUncompress => _httpClient.autoUncompress;

  @override
  set autoUncompress(bool value) => _httpClient.autoUncompress = value;

  @override
  Duration? get connectionTimeout => _httpClient.connectionTimeout;

  @override
  set connectionTimeout(Duration? value) =>
      _httpClient.connectionTimeout = value;

  @override
  Duration get idleTimeout => _httpClient.idleTimeout;

  @override
  set idleTimeout(Duration value) => _httpClient.idleTimeout = value;

  @override
  int? get maxConnectionsPerHost => _httpClient.maxConnectionsPerHost;

  @override
  set maxConnectionsPerHost(int? value) =>
      _httpClient.maxConnectionsPerHost = value;

  @override
  String? get userAgent => _httpClient.userAgent;

  @override
  set userAgent(String? value) => _httpClient.userAgent = value;

  @override
  void addCredentials(
          Uri url, String realm, HttpClientCredentials credentials) =>
      _httpClient.addCredentials(url, realm, credentials);

  @override
  void addProxyCredentials(String host, int port, String realm,
          HttpClientCredentials credentials) =>
      _httpClient.addProxyCredentials(host, port, realm, credentials);

  @override
  set authenticate(
          Future<bool> Function(Uri url, String scheme, String? realm)? f) =>
      _httpClient.authenticate = f;

  @override
  set authenticateProxy(
          Future<bool> Function(
                  String host, int port, String scheme, String? realm)?
              f) =>
      _httpClient.authenticateProxy = f;

  @override
  set badCertificateCallback(
          bool Function(X509Certificate cert, String host, int port)?
              callback) =>
      _httpClient.badCertificateCallback = callback;

  @override
  set findProxy(String Function(Uri url)? f) => _httpClient.findProxy = f;
}
