import '../bonree_agent_impl.dart';
import 'package:flutter/widgets.dart';

abstract class Bonree {
  ///Factory which creates a [Bonree] object
  factory Bonree() => BonreeImpl();

  //bonree flutter start function
  Future<void> start(Widget topLevelWidget);

  /// 设置用户ID
  Future<void> setUserID(String userID);

  /// 设置附加信息
  /// @param extraInfo 附加信息为Key-Value形式 Value限制String或数字类型
  Future<void> setExtraInfo(Map<String, dynamic> extraInfo);

  /// 自定义异常收集
  /// @param exceptionType 异常类型
  /// @param causedBy 异常原因
  /// @param errorDump 异常堆栈
  Future<void> setCustomException(
      String exceptionType, String causedBy, String errorDump);

  /// 自定义视图——标记视图开始（和视图结束方法成对调用）
  /// @param pageName 视图名称 (用于视图匹配，请保证唯一性)
  /// @param pageAlias 视图别名
  Future<void> setCustomPageStart(String pageName, String pageAlias);

  /// 自定义视图——标记视图结束（和视图开始方法成对调用）
  /// @param pageName 视图名称 (用于视图匹配，请保证唯一性)
  /// @param pageAlias 视图别名
  Future<void> setCustomPageEnd(String pageName, String pageAlias);

  /// 自定义事件
  /// @param eventID 事件ID
  /// @param eventName 事件名称
  Future<void> setCustomEvent(String eventID, String eventName, String param);

  /// 自定义日志
  /// @param logInfo 日志信息
  /// @param param 日志附加信息
  Future<void> setCustomLog(String logInfo, String param);

  /// 自定义指标
  /// @param metricName 指标名称
  /// @param metricValue 指标值（整型）
  /// @param param 指标附加信息
  Future<void> setCustomMetric(
      String metricName, int metricValue, String param);

  /// 自定义方法开始
  /// @param methodName 方法名
  /// @param param 附加信息
  Future<void> setCustomMethodStart(String methodName, String param);

  /// 自定义方法结束
  /// @param methodName 方法名
  /// @param param 附加信息
  Future<void> setCustomMethodEnd(String methodName, String param);

  /// 自定义冷启动结束点
  Future<void> recordCustomLaunchEnd();

  /// 错误回调接口
  /// @param callback 回调方法
  ///     @param error 错误原因
  ///     @param stack 错误堆栈
  ///     @param isAsync 是否为异步错误
  void onBonreeErrorCallback(
      callback(Object error, StackTrace stack, bool isAsync));
}
