import 'dart:io';

import 'package:bonree_flutter_plugin/src/agent/bonree_agent_impl.dart';
import 'package:bonree_flutter_plugin/src/business/bonree_method_channel.dart';
import 'package:bonree_flutter_plugin/src/business/util/logger.dart';

import 'bonree_business_interface.dart';
import 'model/bonree_network_model.dart';

class BonreeBusiness implements BonreeBusinessInterface {
  static BonreeBusinessInterface? _instance;
  final Map<String, BonreeNetworkModel> _networkModelMap;

  factory BonreeBusiness() {
    if (_instance == null) {
      _instance = BonreeBusiness.private(new Map());
    }
    return _instance as BonreeBusiness;
  }

  BonreeBusiness.private(this._networkModelMap);

  bool _isStarted() {
    return BonreeImpl().isStarted();
  }

  //崩溃回调
  Future<void> reportCrash(
      String errorName, String reason, String stacktrace) async {
    //获取当前时间戳
    int nowTime = new DateTime.now().millisecondsSinceEpoch;
    await onMethodChannelInvoke("bonree_reportCrash", <String, dynamic>{
      "time": nowTime,
      "errorValue": errorName,
      "reason": reason,
      "stacktrace": stacktrace
    });
  }

  void startWebRequest(String requestKey, String url) {
    BonreeNetworkModel model = new BonreeNetworkModel();
    model.setRequestUrl = url;
    model.setStartTimeMs = DateTime.now().millisecondsSinceEpoch;
    _networkModelMap[requestKey] = model;
  }

  void endWebRequest(
      String requestKey, String requestMethod, HttpHeaders requestHeader) {
    int nowTime = new DateTime.now().millisecondsSinceEpoch;
    BonreeNetworkModel? model = _networkModelMap[requestKey];
    if (model != null) {
      model.setMethod = requestMethod;
      if (nowTime > model.startTimeMs)
        model.setConnectTimeMs = nowTime - model.startTimeMs;

      Map header = new Map();
      requestHeader.forEach((name, values) {
        header[name] = values.join(",");
      });
      model.setHttpRequestHeader = header;
    }
  }

  void startWebResposne(String requestKey) {
    int nowTime = new DateTime.now().millisecondsSinceEpoch;
    BonreeNetworkModel? model = _networkModelMap[requestKey];
    if (model != null) {
      model.setResponseStartTimeMs = nowTime;
    }
  }

  void endWebResponse(String requestKey, int errorCode, int responseDataSize,
      HttpHeaders responseHeader) {
    int nowTime = new DateTime.now().millisecondsSinceEpoch;
    BonreeNetworkModel? model = _networkModelMap[requestKey];
    if (model != null) {
      model.setErrorCode = errorCode;
      model.setResponseDataSize = responseDataSize;
      
      Map header = new Map();
      responseHeader.forEach((name, values) {
        header[name] = values.join(",");
      });
      model.setHttpResponseHeader = header;
      model.setResourceType = responseHeader.contentType.toString();

      if (nowTime > model.responseStartTimeMs)
        model.setResponseTimeMs = nowTime - model.responseStartTimeMs;
    }
  }

  void finishWebRequset(String requestKey, String? errorMsg) {
    BonreeNetworkModel? model = _networkModelMap[requestKey];
    //上报网络数据
    if (model != null) {
      model.setErrorMessage = errorMsg;
      reportNetwork(model, requestKey);
    }
  }

  Future<void> reportNetwork(
      BonreeNetworkModel networkModel, String requestKey) async {
    //获取当前时间戳
    await onMethodChannelInvoke("bonree_reportNetwork", networkModel.toJson());
    _networkModelMap.remove(requestKey);
  }

  @override
  void reportView(int time, String viewId, int loadTime, int model, String? name,
      String method) async {
    if (!_isStarted()) {
      Logger().i("Bonree().start() method was not called!");
      return;
    }
    Logger().i(
        "reportView time:$time, viewId:$viewId, loadTime:$loadTime, model:$model, name:$name, method:$method");
    await onMethodChannelInvoke("bonree_reportView", {
      "time": time,
      "viewId": viewId,
      "loadTime": loadTime,
      "model": model,
      "name": name,
      "method": method
    });
  }
}
